#include "common.h"
#include "Triangle.h"
#include "timing.h"

#if 0
Triangle::Triangle(const Vector3 &p0, const Vector3 &p1, const Vector3 &p2)
{
	m_kP[0] = p0;
	m_kP[1] = p1;
	m_kP[2] = p2;
}
#endif


bool Triangle::hit(Photon &p, HitRecord &hit, float tmin, float tmax) const
{
	float tval;
	START_TIMING("Triangle::hit")

	float A = m_kP[0].x - m_kP[1].x;
	float B = m_kP[0].y - m_kP[1].y;
	float C = m_kP[0].z - m_kP[1].z;

	float D = m_kP[0].x - m_kP[2].x;
	float E = m_kP[0].y - m_kP[2].y;
	float F = m_kP[0].z - m_kP[2].z;
	
	float G = p.m_fDirX;
	float H = p.m_fDirY;
	float I = p.m_fDirZ;

	float J = m_kP[0].x - p.m_fOriginX;
	float K = m_kP[0].y - p.m_fOriginY;
	float L = m_kP[0].z - p.m_fOriginZ;

	float EIHF = E * I - H * F;
	float GFDI = G * F - D * I;
	float DHEG = D * H - E * G;

	float denom = (A * EIHF + B * GFDI + C * DHEG);
	//if(denom < 0) denom = -denom;

	float beta = (J * EIHF + K * GFDI + L * DHEG) / denom;

	if(beta < 0.0f || beta > 1.0f) 
		return false;

	float AKJB = A*K - J*B;
	float JCAL = J*C - A*L;
	float BLKC = B*L - K*C;

	float gamma = (I * AKJB + H * JCAL + J * BLKC) / denom;

	if(gamma < 0.0f || beta + gamma > 1.0f)
		return false;

	tval = -(F * AKJB + E * JCAL + D * BLKC) / denom;

	if(tval >= tmin && tval <= tmax)
	{
		hit.m_fTval = tval;
		float alpha = 1 - gamma - beta;
		hit.m_fAlpha = alpha;
		hit.m_fBeta = beta;
		hit.m_fGamma = gamma;
		return true;
	}

	return false;
	END_TIMING
}