#include "common.h"
#include "geometry.h"

void createPlane(unsigned int lodLevel, unsigned int number, float *&tempVertex, float *&tempWallTexcoordCover, float *&tempNormal, Orientation orientation, float positive, float fScale = 1)
{
	unsigned int i, j;
	float size = (GRID_SIZE * pow(2.0f, (float)MAX_LOD - lodLevel)) * fScale;
	float normal = (positive>=0)?1:-1;

	switch(orientation)
	{
	// x axis
	case 0:
		for(i=0; i<number + 1; i++)
		{
			for(j=0; j<number + 1; j++)
			{
				// vertex coords
				*tempVertex = positive * (size/2.0f) * pow(2.0f, (float)lodLevel);
				tempVertex++;
				*tempVertex = i * size - ((float)number * size/2.0f);
				tempVertex++;
				*tempVertex = j * size - ((float)number * size/2.0f);
				tempVertex++;
				// tex coords
				*tempWallTexcoordCover = (float)i / (float)number;
				tempWallTexcoordCover++;
				*tempWallTexcoordCover = (float)j / (float)number;
				tempWallTexcoordCover++;
				// normals
				*tempNormal = normal;
				tempNormal++;
				*tempNormal = 0;
				tempNormal++;
				*tempNormal = 0;
				tempNormal++;
			}
		}
		break;
	// y axis
	case 1:
		for(i=0; i<number + 1; i++)
		{
			for(j=0; j<number + 1; j++)
			{
				// vertex coords
				*tempVertex = i * size - ((float)number * size/2.0f);
				tempVertex++;
				*tempVertex = positive * (size/2.0f) * pow(2.0f, (float)lodLevel);
				tempVertex++;
				*tempVertex = j * size - ((float)number * size/2.0f);
				tempVertex++;
				// tex coords
				*tempWallTexcoordCover = (float)i / (float)number;
				tempWallTexcoordCover++;
				*tempWallTexcoordCover = (float)j / (float)number;
				tempWallTexcoordCover++;
				// normals
				*tempNormal = 0;
				tempNormal++;
				*tempNormal = normal;
				tempNormal++;
				*tempNormal = 0;
				tempNormal++;
			}
		}
		break;
	// z axis
	case 2:
		for(i=0; i<number + 1; i++)
		{
			for(j=0; j<number + 1; j++)
			{
				// vertex coords
				*tempVertex = i * size - ((float)number * size/2.0f);
				tempVertex++;
				*tempVertex = j * size - ((float)number * size/2.0f);
				tempVertex++;
				*tempVertex = positive * (size/2.0f) * pow(2.0f, (float)lodLevel);
				tempVertex++;
				// tex coords
				*tempWallTexcoordCover = (float)i / (float)number;
				tempWallTexcoordCover++;
				*tempWallTexcoordCover = (float)j / (float)number;
				tempWallTexcoordCover++;
				// normals
				*tempNormal = 0;
				tempNormal++;
				*tempNormal = 0;
				tempNormal++;
				*tempNormal = normal;
				tempNormal++;
			}
		}
		break;
	}
}

void createBox(Mesh *pMesh, int lodLevel, float fScale, float fTransX, float fTransY, float fTransZ)
{
	unsigned int number = (unsigned int)pow(2.0f, (float)lodLevel);
	unsigned int maxNumber = (unsigned int)pow(2.0f, (float)MAX_LOD);
	unsigned int i, j, k;

	pMesh->m_iNumberTriangles = number * number * 2 * 6;
	pMesh->m_iNumberVertices = (number + 1) * (number + 1) * 6;
	printf("Creating box: Lod level: %d, Number of traingles: %d\n", lodLevel, pMesh->m_iNumberTriangles);

	if(pMesh->m_pIndex !=0)
	{
		free(pMesh->m_pIndex);
		free(pMesh->m_pVertex);
		free(pMesh->m_pTexCoord);
		free(pMesh->m_pNormal);
	}

	pMesh->m_pVertex = (float *)malloc((maxNumber + 1) * (maxNumber + 1) * sizeof(float) * 3 * 6);
	float *tempVertex = pMesh->m_pVertex;
	pMesh->m_pTexCoord = (float *)malloc((maxNumber + 1) * (maxNumber + 1) * sizeof(float) * 2 * 6);
	float *tempWallTexcoordCover = pMesh->m_pTexCoord;
	pMesh->m_pNormal = (float *)malloc((maxNumber + 1) * (maxNumber + 1) * sizeof(float) * 3 * 6);
	float *tempNormal = pMesh->m_pNormal;

	// positive X face
	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, X_AXIS, 1, fScale);
	// negative X face
	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, X_AXIS, -1, fScale);
	// positive Y face
	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, Y_AXIS, 1, fScale);
	// negative Y face
	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, Y_AXIS, -1, fScale);
	// positive Z face
	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, Z_AXIS, 1, fScale);
	// negative Z face
	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, Z_AXIS, -1, fScale);

	pMesh->m_pIndex = (unsigned int *)malloc(maxNumber * maxNumber * sizeof(unsigned int) * 6 * 6);
	unsigned int *temp_index = pMesh->m_pIndex;

	unsigned int n = 0;
	for(k=0; k<6; k++)
	{
		for(i=0; i<number; i++)
		{
			for(j=0; j<number; j++)
			{
				*temp_index = i * (number + 1) + j + n;
				temp_index++;
				*temp_index = (i + 1) * (number + 1) + j + n;
				temp_index++;
				*temp_index = i * (number + 1) + j + 1 + n;
				temp_index++;
			
				*temp_index = i * (number + 1) + j + 1 + n;
				temp_index++;
				*temp_index = (i + 1) * (number + 1) + j + 1 + n;
				temp_index++;
				*temp_index = (i + 1) * (number + 1) + j + n;
				temp_index++;
			}
		}
		n += (number + 1) * (number + 1);
	}
}

// creates the wall mesh based on the LOD level given
void createWall(Mesh *pMesh, int lodLevel)
{
	int numberWalls = 2;
	unsigned int number = (unsigned int)pow(2.0f, (float)lodLevel);
	unsigned int maxNumber = (unsigned int)pow(2.0f, (float)MAX_LOD);
	unsigned int i, j, k;

	pMesh->m_iNumberTriangles = number * number * 2 * numberWalls;
	pMesh->m_iNumberVertices = (number + 1) * (number + 1) * numberWalls;
	printf("Create wall: Lod level: %d, Triangles: %d, Vertices: %d\n", 
		lodLevel, pMesh->m_iNumberTriangles, pMesh->m_iNumberVertices);

	if(pMesh->m_pIndex !=0)
	{
		free(pMesh->m_pIndex);
		free(pMesh->m_pVertex);
		free(pMesh->m_pTexCoord);
		free(pMesh->m_pNormal);
	}

	pMesh->m_pVertex = (float *)malloc(pMesh->m_iNumberVertices * sizeof(float) * 3);
	float *tempVertex = pMesh->m_pVertex;
	pMesh->m_pTexCoord = (float *)malloc(pMesh->m_iNumberVertices * sizeof(float) * 2);
	float *tempWallTexcoordCover = pMesh->m_pTexCoord;
	pMesh->m_pNormal = (float *)malloc(pMesh->m_iNumberVertices * sizeof(float) * 3);
	float *tempNormal = pMesh->m_pNormal;

	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, Y_AXIS, -0.5, 0.3f);
	createPlane(lodLevel, number, tempVertex, tempWallTexcoordCover, tempNormal, Y_AXIS, 1);

	pMesh->m_pIndex = (unsigned int *)malloc(pMesh->m_iNumberTriangles* sizeof(unsigned int) * 3);
	unsigned int *temp_index = pMesh->m_pIndex;

	unsigned int n = 0;
	for(k=0; k<numberWalls; k++)
	{
		for(i=0; i<number; i++)
		{
			for(j=0; j<number; j++)
			{
				*temp_index = i * (number + 1) + j + n;
				temp_index++;
				*temp_index = (i + 1) * (number + 1) + j + n;
				temp_index++;
				*temp_index = i * (number + 1) + j + 1 + n;
				temp_index++;

				*temp_index = i * (number + 1) + j + 1 + n;
				temp_index++;
				*temp_index = (i + 1) * (number + 1) + j + 1 + n;
				temp_index++;
				*temp_index = (i + 1) * (number + 1) + j + n;
				temp_index++;
			}
		}
		n += (number + 1) * (number + 1);
	}
}

void createQuad(Mesh *pMesh)
{
	unsigned int i;

	pMesh->m_iNumberTriangles = 2;
	pMesh->m_iNumberVertices = 6;

	pMesh->m_pVertex = (float *)malloc(pMesh->m_iNumberVertices * sizeof(float) * 3);
	float *tempVertex = pMesh->m_pVertex;
	pMesh->m_pTexCoord = (float *)malloc(pMesh->m_iNumberVertices * sizeof(float) * 2);
	float *tempTexcoord = pMesh->m_pTexCoord;
	pMesh->m_pNormal = (float *)malloc(pMesh->m_iNumberVertices * sizeof(float) * 3);
	float *tempNormal = pMesh->m_pNormal;

	tempVertex[0] = -8.0f;
	tempVertex[1] = 8.0f;
	tempVertex[2] = 0.0f;
	tempVertex += 3;
	tempVertex[0] = -8.0f;
	tempVertex[1] = -8.0f;
	tempVertex[2] = 0.0f;
	tempVertex += 3;
	tempVertex[0] = 8.0f;
	tempVertex[1] = -8.0f;
	tempVertex[2] = 0.0f;
	tempVertex += 3;

	tempVertex[0] = -8.0f;
	tempVertex[1] = 8.0f;
	tempVertex[2] = 0.0f;
	tempVertex += 3;
	tempVertex[0] = 8.0f;
	tempVertex[1] = -8.0f;
	tempVertex[2] = 0.0f;
	tempVertex += 3;
	tempVertex[0] = 8.0f;
	tempVertex[1] = 8.0f;
	tempVertex[2] = 0.0f;

	tempTexcoord[0] = 0;
	tempTexcoord[1] = 1;
	tempTexcoord += 2;
	tempTexcoord[0] = 0;
	tempTexcoord[1] = 0;
	tempTexcoord += 2;
	tempTexcoord[0] = 1;
	tempTexcoord[1] = 0;
	tempTexcoord += 2;
	tempTexcoord[0] = 0;
	tempTexcoord[1] = 1;
	tempTexcoord += 2;
	tempTexcoord[0] = 1;
	tempTexcoord[1] = 0;
	tempTexcoord += 2;
	tempTexcoord[0] = 1;
	tempTexcoord[1] = 1;

	for(i=0; i<6; i++)
	{
		tempNormal[0] = 0;
		tempNormal[1] = 0;
		tempNormal[2] = 1;
		tempNormal += 3;
	}

	pMesh->m_pIndex = (unsigned int *)malloc(pMesh->m_iNumberTriangles * sizeof(unsigned int) * 3);
	unsigned int *temp_index = pMesh->m_pIndex;

	for(i=0; i<6; i++)
		temp_index[i] = i;
}
