#include "common.h"
#include "photon.h"
#include "geometry.h"
#include "meshmanager.h"

bool g_bToggleFPS;

int g_uiWindowWidth;
int g_uiWindowHeight;
float g_fWinAspectRatio;
float g_fDistance = -20;
float g_fAngle = 0;
int g_iNumberPhotons = 1;
int g_iNumberLights = 1;
int g_iLODLevel = 2;
int g_iNumberTracePasses = 1;
float g_fFPS = 30;

// Options
bool g_bToggleWireframe = true;
bool g_bShowPhotonDensity = true;
bool g_bShowPhotonTrace = true;
bool g_bPauseMode = false;
bool g_bShowHUD = true;
bool g_bToggleAnimate = false;
bool g_bToggleGeneratePhotons = true;

unsigned int g_iPhotonIDMap, g_iPhotonDirMap, g_iPhotonPowerMap;
PhotonMap *g_pPhotonMap;
Light *g_pLight;
MeshManager *g_pMeshManager;

void keyboardFunction(unsigned char c)
{
	switch(c)
	{
	case 27:
		exit(0);
		break;
	case 'w':
	case 'W':
		g_fDistance++;
		break;
	case 's':
	case 'S':
		g_fDistance--;
		break;	
	case 'a':
	case 'A':
		g_fAngle += 3;
		break;
	case 'd':
	case 'D':
		g_fAngle -= 3;
		break;
	case 'i':
	case 'I':
		g_pLight->m_fY += 1;
		break;
	case 'k':
	case 'K':
		g_pLight->m_fY -= 1;
		break;	
	case 'j':
	case 'J':
		g_pLight->m_fX -= 1;
		break;
	case 'l':
	case 'L':
		g_pLight->m_fX += 1;
		break;	
	case 'h':
	case 'H':
		g_bShowHUD = !g_bShowHUD;
		break;
	case 'p':
	case 'P':
		g_bShowPhotonDensity = !g_bShowPhotonDensity;
		g_bToggleWireframe = g_bShowPhotonDensity || g_bShowPhotonTrace;
		break;
	case 't':
	case 'T':
		g_bShowPhotonTrace = !g_bShowPhotonTrace;
		g_bToggleWireframe = g_bShowPhotonDensity || g_bShowPhotonTrace;
		break;
	case ' ':
		g_bPauseMode = !g_bPauseMode;
		break;
	case 'r':
	case 'R':
		g_bToggleAnimate = !g_bToggleAnimate;
		break;	
	case 'g':
	case 'G':
		g_bToggleGeneratePhotons = !g_bToggleGeneratePhotons;
		break;
	}
}

void keyboardGLUT(unsigned char c, int x, int y)
{
	keyboardFunction(c);
	glutPostRedisplay();
}

void idleGLUT(void)
{
	Sleep(10);
	glutPostRedisplay();	
}

void mouseGLUT(int button, int state, int x, int y)
{
	glutPostRedisplay();
}

void render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,g_fWinAspectRatio, 0.1f, 1000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	if(g_bToggleAnimate)
	{
		g_fAngle++;
	}

	glTranslatef(0,0,g_fDistance);
	glRotatef(g_fAngle, 0,1,0);

	if(g_bToggleWireframe)
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	drawHUD();

	for(int i=0; i<g_pMeshManager->getNumberMeshes(); i++)
		drawObjectCg(g_pMeshManager->getMesh(i));

	if(g_bShowPhotonDensity)
		drawPhotonDensity();
	if(g_bShowPhotonTrace)
		drawPhotonTrace();
}

void displayGLUT()
{
	if(!g_bPauseMode)
		algorithm();

	START_TIMING("Render")
	render();

	glutSwapBuffers();
	calculateFPS();
	END_TIMING
}

void updateWindowTitleGLUT()
{
	char *title = "Photon Mapping";
	glutSetWindowTitle(title);
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	g_uiWindowWidth  = w;
	g_uiWindowHeight = h;
	g_fWinAspectRatio = (double) w / (double) h;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,g_fWinAspectRatio, 0.1f, 1000);
}

void reshapeGLUT(int w, int h)
{
	reshape(w, h);
	updateWindowTitleGLUT();
}

void select(int item)
{
	switch(item)
	{
	case MENU_QUIT:
		exit(0);
		break;
	case MENU_WIREFREAME_TOGGLE:
		g_bToggleWireframe = !g_bToggleWireframe;
		break;
	case MENU_SHOW_PHOTON_DENSITY:
		g_bShowPhotonDensity = !g_bShowPhotonDensity;
		g_bToggleWireframe = g_bShowPhotonDensity || g_bShowPhotonTrace;
		break;
	case MENU_SHOW_PHOTON_TRACE:
		g_bShowPhotonTrace = !g_bShowPhotonTrace;
		g_bToggleWireframe = g_bShowPhotonDensity || g_bShowPhotonTrace;
		break;
	case MENU_PAUSE_MODE:
		g_bPauseMode = !g_bPauseMode;
		break;
	case MENU_PRINT_ID_BUFFER:
		g_pPhotonMap->printIDTexture();
		break;
	case MENU_PRINT_POW_BUFFER:
		g_pPhotonMap->printPowTexture();
		break;
	case MENU_PRINT_DIR_BUFFER:
		g_pPhotonMap->printDirTexture();
		break;
	case MENU_NUMBER_PHOTONS_1:
		g_pPhotonMap->setNumberPhotons(1);
		break;
	case MENU_NUMBER_PHOTONS_64:
		g_pPhotonMap->setNumberPhotons(64);
		break;
	case MENU_NUMBER_PHOTONS_256:
		g_pPhotonMap->setNumberPhotons(256);
		break;
	}
	glutPostRedisplay();
}

void initMenus()
{
	int debugMenu = glutCreateMenu(select);
	glutAddMenuEntry("Print ID Texture", MENU_PRINT_ID_BUFFER);
	glutAddMenuEntry("Print Pow Texture", MENU_PRINT_POW_BUFFER);
	glutAddMenuEntry("Print Dir Texture", MENU_PRINT_DIR_BUFFER);

	int photonMenu = glutCreateMenu(select);
	glutAddMenuEntry("Number of Photons: 1", MENU_NUMBER_PHOTONS_1);
	glutAddMenuEntry("Number of Photons: 64", MENU_NUMBER_PHOTONS_64);
	glutAddMenuEntry("Number of Photons: 256", MENU_NUMBER_PHOTONS_256);


	glutCreateMenu(select);
	glutAddMenuEntry("Wireframe\t", MENU_WIREFREAME_TOGGLE);
	glutAddMenuEntry("Toggle Photon Density\t[P]", MENU_SHOW_PHOTON_DENSITY);
	glutAddMenuEntry("Toggle Photon Trace\t[T]", MENU_SHOW_PHOTON_TRACE);
	glutAddMenuEntry("Toggle Pause Mode\t[Space]", MENU_PAUSE_MODE);
	glutAddMenuEntry("Quit\t[Esc]", MENU_QUIT);
	glutAddSubMenu("Photon Setup", photonMenu);
	glutAddSubMenu("Debug", debugMenu);

	glutAttachMenu(GLUT_RIGHT_BUTTON);
}

void initGL()
{
#ifndef SN_TARGET_PS3
	initExtensions();
#endif

	glClearStencil(0);
	glClearColor(0.1f,0.1f,0.5f, 1.0f);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60,g_fWinAspectRatio, 0.1f, 1000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(50,100,-10,0,0,0,0,1,0);

	glEnable(GL_DEPTH_TEST);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
//	glCullFace(GL_FRONT);
//	glEnable(GL_CULL_FACE);

	glGenTextures(1, &g_iPhotonIDMap);
	glGenTextures(1, &g_iPhotonDirMap);
	glGenTextures(1, &g_iPhotonPowerMap);
}

void initApp()
{	
	initCg();
	//g_pLight = new Light(100,100,100,1,1,-2,POINT_LIGHT);
	g_pLight = new Light(100, 100, 100, 1, 1, -2, CONE_LIGHT);
	
	g_pPhotonMap = new PhotonMap(1, &g_pLight, g_iNumberPhotons);

	g_pMeshManager = new MeshManager();

	Mesh *pMesh = g_pMeshManager->createMesh();
	//createWall(pMesh, g_iLODLevel);
	createBox(pMesh, g_iLODLevel, 1, 0,0,0);
	g_pMeshManager->addMesh(pMesh);

/*	Mesh *pMesh1 = g_pMeshManager->createMesh();
	createBox(pMesh1, g_iLODLevel, 0.3f, 0,0,0);
	g_pMeshManager->addMesh(pMesh1);*/

	g_pMeshManager->deIndex(0);
	//g_pMeshManager->deIndex(1);

	g_pPhotonMap->addMesh(g_pMeshManager->getMesh(0));
	//g_pPhotonMap->addMesh(g_pMeshManager->getMesh(1));
}

int main(int argc, char **argv)
{

	g_uiWindowWidth = 640;
	g_uiWindowHeight = 480;
	g_fWinAspectRatio = g_uiWindowWidth/(float)g_uiWindowHeight;

	glutInitWindowSize(g_uiWindowWidth, g_uiWindowHeight);
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);

	glutCreateWindow("Photon Mapping");

	glutDisplayFunc(displayGLUT);
	glutKeyboardFunc(keyboardGLUT);
	glutReshapeFunc(reshapeGLUT);
	glutIdleFunc(idleGLUT);
	glutMouseFunc(mouseGLUT);

	initGL();
	initApp();
	initMenus();

	glutMainLoop();
	return 0;
}