#include "common.h"
#include "meshmanager.h"
#include "vector.h"


MeshManager::MeshManager()
{
	m_iNumberMeshes = 0;
}

void MeshManager::addMesh(Mesh *pMesh)
{
	m_pMeshes[m_iNumberMeshes] = pMesh;
	m_iNumberMeshes++;
}

Mesh *MeshManager::getMesh(int id)
{
	return m_pMeshes[id];
}

Mesh *MeshManager::createMesh()
{
	Mesh *pMesh = new Mesh;
	memset(pMesh, 0, sizeof(Mesh));

	return pMesh;
}

Mesh *MeshManager::deIndex(Mesh *pMesh)
{
	unsigned int i;

	Mesh *pNonIndexMesh = new Mesh();

	pNonIndexMesh->m_iNumberTriangles = pMesh->m_iNumberTriangles;
	pNonIndexMesh->m_iNumberVertices = pMesh->m_iNumberTriangles * 3;
	pNonIndexMesh->m_pIndex = new unsigned int[pNonIndexMesh->m_iNumberVertices];
	pNonIndexMesh->m_pVertex = new float[pNonIndexMesh->m_iNumberVertices * 3];
	pNonIndexMesh->m_pTexCoord = new float[pNonIndexMesh->m_iNumberVertices * 2];
	pNonIndexMesh->m_pNormal = new float[pNonIndexMesh->m_iNumberVertices * 3];

	for(i=0; i<pMesh->m_iNumberTriangles; i++)
	{
		pNonIndexMesh->m_pIndex[3*i] = 3*i;
		pNonIndexMesh->m_pIndex[3*i+1] = 3*i+1;
		pNonIndexMesh->m_pIndex[3*i+2] = 3*i+2;
	}

		// de-index now
	for(i=0; i<pMesh->m_iNumberTriangles; i++)
	{
		Vector3 v1(pMesh->m_pVertex[3*pMesh->m_pIndex[3*i]], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i]+1], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i]+2]);
		Vector3 v2(pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+1]], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+1]+1], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+1]+2]);
		Vector3 v3(pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+2]], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+2]+1], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+2]+2]);

		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i]] = v1.x;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i]+1] = v1.y;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i]+2] = v1.z;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i+1]] = v2.x;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i+1]+1] = v2.y;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i+1]+2] = v2.z;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i+2]] = v3.x;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i+2]+1] = v3.y;
		pNonIndexMesh->m_pVertex[3*pNonIndexMesh->m_pIndex[3*i+2]+2] = v3.z;

		Vector3 t1(pMesh->m_pNormal[2*pMesh->m_pIndex[3*i]], 
			pMesh->m_pNormal[3*pMesh->m_pIndex[3*i]+1], 
			0);
		Vector3 t2(pMesh->m_pNormal[2*pMesh->m_pIndex[3*i+1]], 
			pMesh->m_pNormal[2*pMesh->m_pIndex[3*i+1]+1], 
			0);
		Vector3 t3(pMesh->m_pNormal[2*pMesh->m_pIndex[3*i+2]], 
			pMesh->m_pNormal[2*pMesh->m_pIndex[3*i+2]+1], 
			0);
		pNonIndexMesh->m_pTexCoord[2*pNonIndexMesh->m_pIndex[3*i]] = t1.x;
		pNonIndexMesh->m_pTexCoord[2*pNonIndexMesh->m_pIndex[3*i]+1] = t1.y;
		pNonIndexMesh->m_pTexCoord[2*pNonIndexMesh->m_pIndex[3*i+1]] = t2.x;
		pNonIndexMesh->m_pTexCoord[2*pNonIndexMesh->m_pIndex[3*i+1]+1] = t2.y;
		pNonIndexMesh->m_pTexCoord[2*pNonIndexMesh->m_pIndex[3*i+2]] = t3.x;
		pNonIndexMesh->m_pTexCoord[2*pNonIndexMesh->m_pIndex[3*i+2]+1] = t3.y;

		Vector3 n1(pMesh->m_pNormal[3*pMesh->m_pIndex[3*i]], 
			pMesh->m_pNormal[3*pMesh->m_pIndex[3*i]+1], 
			pMesh->m_pNormal[3*pMesh->m_pIndex[3*i]+2]);
		Vector3 n2(pMesh->m_pNormal[3*pMesh->m_pIndex[3*i+1]], 
			pMesh->m_pNormal[3*pMesh->m_pIndex[3*i+1]+1], 
			pMesh->m_pNormal[3*pMesh->m_pIndex[3*i+1]+2]);
		Vector3 n3(pMesh->m_pNormal[3*pMesh->m_pIndex[3*i+2]], 
			pMesh->m_pNormal[3*pMesh->m_pIndex[3*i+2]+1], 
			pMesh->m_pNormal[3*pMesh->m_pIndex[3*i+2]+2]);
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i]] = n1.x;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i]+1] = n1.y;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i]+2] = n1.z;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i+1]] = n2.x;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i+1]+1] = n2.y;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i+1]+2] = n2.z;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i+2]] = n3.x;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i+2]+1] = n3.y;
		pNonIndexMesh->m_pNormal[3*pNonIndexMesh->m_pIndex[3*i+2]+2] = n3.z;
	}

	return pNonIndexMesh;
}

void MeshManager::deIndex(int iID)
{
	Mesh *pNonIndexMesh = deIndex(getMesh(iID));

	delete m_pMeshes[iID];
	m_pMeshes[iID] = pNonIndexMesh;
}