#include "common.h"
#include "photon.h"
#include "triangle.h"

// Light Class

Light::	Light(float fRed, float fGreen, float fBlue, 
		float fX, float fY, float fZ,
		LightType eType)
{
	// power
	m_fRed = fRed;
	m_fGreen = fGreen;
	m_fBlue = fBlue;

	// position
	m_fX = fX;
	m_fY = fY;
	m_fZ = fZ;

	// type
	m_eType = eType;
}

void Light::emitPhotons(Photon *pPhotons)
{
	switch(m_eType)
	{
	case POINT_LIGHT:
		emitPhotonsDiffuseLight(pPhotons);
		break;
	case CONE_LIGHT:
		emitPhotonsDirectionalLight(pPhotons, 0,0,1, 0.90f);
		break;
	case AREA_LIGHT:
		printf("Light model not supported yet!\n");
		break;
	}
}

void Light::emitPhotonsDiffuseLight(Photon *pPhotons)
{
	int i;
	for(i=0; i<m_iNumberPhotons; i++)
	{
		float x,y,z;
		x = 2*rand01() - 1;
		y = 2*rand01() - 1;
		z = 2*rand01() - 1;

		normalize(x, y, z, pPhotons[i].m_fDirX, pPhotons[i].m_fDirY, pPhotons[i].m_fDirZ);
		pPhotons[i].m_fRed = m_fRed / m_iNumberPhotons;
		pPhotons[i].m_fGreen = m_fGreen / m_iNumberPhotons;
		pPhotons[i].m_fBlue = m_fBlue / m_iNumberPhotons;
		pPhotons[i].m_fOriginX = m_fX;
		pPhotons[i].m_fOriginY = m_fY;
		pPhotons[i].m_fOriginZ = m_fZ;
		pPhotons[i].m_fX = m_fX;
		pPhotons[i].m_fY = m_fY;
		pPhotons[i].m_fZ = m_fZ;
	}
}
void Light::emitPhotonsDirectionalLight(Photon *pPhotons, float fDirX, float fDirY, float fDirZ, float fWide)
{

	int i=0;

	while(i<m_iNumberPhotons)
	{
		float x,y,z;
		x = 2*rand01() - 1;
		y = 2*rand01() - 1;
		z = 2*rand01() - 1;
		float xn, yn, zn;
		normalize(x, y, z, xn, yn, zn);

		if((fDirX * xn + fDirY * yn + fDirZ * zn) > fWide)
		{
			pPhotons[i].m_fDirX = xn;
			pPhotons[i].m_fDirY = yn;
			pPhotons[i].m_fDirZ = zn;
			pPhotons[i].m_fRed = m_fRed / m_iNumberPhotons;
			pPhotons[i].m_fGreen = m_fGreen / m_iNumberPhotons;
			pPhotons[i].m_fBlue = m_fBlue / m_iNumberPhotons;
			pPhotons[i].m_fOriginX = m_fX;
			pPhotons[i].m_fOriginY = m_fY;
			pPhotons[i].m_fOriginZ = m_fZ;
			pPhotons[i].m_fX = m_fX;
			pPhotons[i].m_fY = m_fY;
			pPhotons[i].m_fZ = m_fZ;
			i++;
		}
	}
}

// Photon Class

void Photon::trace(Mesh *pMesh)
{
}

// PhotonMap Class

PhotonMap::PhotonMap(int numberLights, Light **lights, int numberPhotons):
m_iNumberPhotons(numberPhotons),
m_iNumberLights(numberLights)
{
	m_iTextureWidth = sqrt((float)m_iNumberPhotons);
	m_iTextureHeight = sqrt((float)m_iNumberPhotons);
	m_pPhotons = new Photon[m_iNumberPhotons];
	int i;
	m_iNumberLights = numberLights;
	m_iPhotonsPerLight = numberPhotons/numberLights;
	for(i=0; i<m_iNumberLights; i++)
	{
		m_pLights[i] = lights[i];
		m_pLights[i]->setNumberPhotons(m_iPhotonsPerLight);
	}

	// alocate buffers
	m_pPhotonDirBuffer = new char[m_iNumberPhotons*4];
	memset(m_pPhotonDirBuffer, 0, m_iNumberPhotons*4);
	m_pPhotonPowerBuffer = new char[m_iNumberPhotons*4];
	memset(m_pPhotonPowerBuffer, 0, m_iNumberPhotons*4);

	m_pfTexCoordBuffer = 0;
}

PhotonMap::~PhotonMap()
{
	delete m_pPhotons;

	delete m_pPhotonIDBuffer;
	delete m_pPhotonDirBuffer;
	delete m_pPhotonPowerBuffer;
}

void PhotonMap::setNumberPhotons(int iNumberPhotons) 
{
	m_iNumberPhotons = iNumberPhotons;
	m_iTextureWidth = sqrt((float)m_iNumberPhotons);
	m_iTextureHeight = sqrt((float)m_iNumberPhotons);
	delete []m_pPhotons;
	m_pPhotons = new Photon[m_iNumberPhotons];
	int i;
	m_iPhotonsPerLight = m_iNumberPhotons/m_iNumberLights;
	for(i=0; i<m_iNumberLights; i++)
	{
		m_pLights[i]->setNumberPhotons(m_iPhotonsPerLight);
	}

	// alocate buffers
	delete []m_pPhotonDirBuffer;
	m_pPhotonDirBuffer = new char[m_iNumberPhotons*4];
	memset(m_pPhotonDirBuffer, 0, m_iNumberPhotons*4);
	delete []m_pPhotonPowerBuffer;
	m_pPhotonPowerBuffer = new char[m_iNumberPhotons*4];
	memset(m_pPhotonPowerBuffer, 0, m_iNumberPhotons*4);

}


void PhotonMap::generatePhotons()
{
	int i;
//	static bool done = false;
//	if(done) return;
//	done = true;
	for(i=0; i<m_iNumberLights; i++)
	{
		m_pLights[i]->emitPhotons(&m_pPhotons[i*m_iPhotonsPerLight]);
	}
}

void PhotonMap::trace(Mesh *pMesh, unsigned int iNPasses)
{
	// Take each photon, find closest intersection with the geometry then
	// store its new position at the intersection point
	START_TIMING("PhotonMap::trace")
	int i, j;
	// for all the photons
	for(i=0; i<m_iNumberPhotons; i++)
	{
		// for each triangle
		bool bAnyHit = false;
		unsigned int iTriangleID = -1;
		Vector3 vHit1, vHit2, vHit3;
		HitRecord kClosestHit = {10000,10000,10000,10000};
		for(j=0; j<pMesh->m_iNumberTriangles; j++)
		{
			int id1 = pMesh->m_pIndex[3*j];
			Vector3 v1(pMesh->m_pVertex[3*id1], 
				pMesh->m_pVertex[3*id1+1], 
				pMesh->m_pVertex[3*id1+2]);
			id1 = pMesh->m_pIndex[3*j+1];
			Vector3 v2(pMesh->m_pVertex[3*id1], 
				pMesh->m_pVertex[3*id1+1], 
				pMesh->m_pVertex[3*id1+2]);
			id1 = pMesh->m_pIndex[3*j+2];
			Vector3 v3(pMesh->m_pVertex[3*id1], 
				pMesh->m_pVertex[3*id1+1], 
				pMesh->m_pVertex[3*id1+2]);

			Triangle kT(v2, v1, v3);
			HitRecord kHit = {0,0,0,0};
	
			if(kT.hit(m_pPhotons[i], kHit))
			{
				bAnyHit = true;
				if(kHit.m_fTval < kClosestHit.m_fTval)
				{
					kClosestHit = kHit;
					iTriangleID = j;
					vHit1 = v1;
					vHit2 = v2;
					vHit3 = v3;
				}
				break;
			}
		}
		if(bAnyHit)
		{
			float x, y, z;

			x = vHit1.x * kClosestHit.m_fAlpha + vHit2.x * kClosestHit.m_fBeta + vHit3.x * kClosestHit.m_fGamma;
			y = vHit1.y * kClosestHit.m_fAlpha + vHit2.y * kClosestHit.m_fBeta + vHit3.y * kClosestHit.m_fGamma;
			z = vHit1.z * kClosestHit.m_fAlpha + vHit2.z * kClosestHit.m_fBeta + vHit3.z * kClosestHit.m_fGamma;

			m_pPhotons[i].m_fX = x;
			m_pPhotons[i].m_fY = y;
			m_pPhotons[i].m_fZ = z;
		}
	}
	END_TIMING
}

void PhotonMap::buildTextures()
{
	int i,j;
	int iMapSize = (int)sqrt((float)m_iNumberPhotons);
	for(i=0; i<iMapSize; i++)
		for(j=0; j<iMapSize; j++)
		{
			Photon *p = getPhoton(i*iMapSize + j);
			m_pPhotonDirBuffer[4*(i*iMapSize + j)] = (char)(p->m_fDirX * 255);
			m_pPhotonDirBuffer[4*(i*iMapSize + j)+1] = (char)(p->m_fDirY * 255);
			m_pPhotonDirBuffer[4*(i*iMapSize + j)+2] = (char)(p->m_fDirZ * 255);

			m_pPhotonPowerBuffer[4*(i*iMapSize + j)] = (char)ceil(p->m_fRed * 255);
			m_pPhotonPowerBuffer[4*(i*iMapSize + j)+1] = (char)ceil(p->m_fGreen * 255);
			m_pPhotonPowerBuffer[4*(i*iMapSize + j)+2] = (char)ceil(p->m_fBlue * 255);
		}
}

void PhotonMap::addMesh(Mesh *pMesh)
{
	int size;

	m_iPhotonIDTextureWidth = (sqrt((float)pMesh->m_iNumberTriangles) + 1);
	m_iPhotonIDTextureWidth *= PHOTONID_SQUARE_SIZE;
	m_iPhotonIDTextureHeight = m_iPhotonIDTextureWidth;
	size = m_iPhotonIDTextureWidth * m_iPhotonIDTextureHeight * 4;

	m_pPhotonIDBuffer = new char[size];
}

void PhotonMap::buildPhotonIdTexture(Mesh *pMesh)
{
	START_TIMING("PhotonMap::buildPhotonIdTexture")
	unsigned int i;
	int j;
	int iX = 1, iY = 0;
	float fTexCoordX = 0, fTexCoordY = 0;

	memset(m_pPhotonIDBuffer, 0, m_iPhotonIDTextureWidth * m_iPhotonIDTextureWidth * 4);

	int iSquaresPerDimension = m_iPhotonIDTextureWidth / sqrt((float)MAX_PHOTONS_PER_TRIANGLE);

	if(m_pfTexCoordBuffer)
		delete []m_pfTexCoordBuffer;
	m_pfTexCoordBuffer = new float[pMesh->m_iNumberVertices * 2];

	char *pBuffer = m_pPhotonIDBuffer;

	// for all triangles
	for(i=0; i<pMesh->m_iNumberTriangles; i++)
	{
		Vector3 v1(pMesh->m_pVertex[3*pMesh->m_pIndex[3*i]], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i]+1], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i]+2]);
		Vector3 v2(pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+1]], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+1]+1], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+1]+2]);
		Vector3 v3(pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+2]], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+2]+1], 
			pMesh->m_pVertex[3*pMesh->m_pIndex[3*i+2]+2]);

		m_pfTexCoordBuffer[2*pMesh->m_pIndex[3*i]] = fTexCoordX;
		m_pfTexCoordBuffer[2*pMesh->m_pIndex[3*i]+1] = fTexCoordY;
		m_pfTexCoordBuffer[2*pMesh->m_pIndex[3*i+1]] = fTexCoordX;
		m_pfTexCoordBuffer[2*pMesh->m_pIndex[3*i+1]+1] = fTexCoordY;
		m_pfTexCoordBuffer[2*pMesh->m_pIndex[3*i+2]] = fTexCoordX;
		m_pfTexCoordBuffer[2*pMesh->m_pIndex[3*i+2]+1] = fTexCoordY;

		unsigned int iArea = (unsigned int)calculateArea(v1, v2, v3);
		float fPhotonDistance;
		Vector3 kCenter = (v1 + v2 + v3)/3;
		fPhotonDistance = kCenter.dist(v1)/* / 2*/;

		int iFoundPhotons = 0;
		int iPatchX = 0, iPatchY = 0;
		// for all triangles or until we find enough photons to 
		// aproximate lighting on this triangle
		for(j=0; j<m_iNumberPhotons && iFoundPhotons<MAX_PHOTONS_PER_TRIANGLE; j++)
		{
			Photon *pPhoton = getPhoton(j);

			Vector3 kP(pPhoton->m_fX, pPhoton->m_fY, pPhoton->m_fZ);

			// NOTE: this can be replaced with distance between point and triangle
#if 0
			if(v1.dist(kP) < fPhotonDistance ||
				v2.dist(kP) < fPhotonDistance ||
				v3.dist(kP) < fPhotonDistance)
#else
			if(kCenter.dist(kP) < fPhotonDistance)
#endif
			{
				// we have found a photon, store its ID
				pBuffer[4*(iPatchY*m_iPhotonIDTextureWidth + iPatchX)] = (j & 0xFF00) >> 8;
				pBuffer[4*(iPatchY*m_iPhotonIDTextureWidth + iPatchX)+1] = j & 0x00FF;
				pBuffer[4*(iPatchY*m_iPhotonIDTextureWidth + iPatchX)+2] = (iArea & 0xFF00) >> 8;
				pBuffer[4*(iPatchY*m_iPhotonIDTextureWidth + iPatchX)+3] = iArea & 0x00FF;
				iFoundPhotons++;
				iPatchX++;
				if(iPatchX>=PHOTONID_SQUARE_SIZE)
				{
					iPatchY++;
					iPatchX = 0;
				}
			}
		}
		// now fill in the id's for the photons that could not be found
		int k,l;
		for(k=iPatchY; k<PHOTONID_SQUARE_SIZE; k++)
		{
			for(l=iPatchX; l<PHOTONID_SQUARE_SIZE; l++)
			{
				pBuffer[4*(k*m_iPhotonIDTextureWidth + l)] = 0;
				pBuffer[4*(k*m_iPhotonIDTextureWidth + l)+1] = 0;
				pBuffer[4*(k*m_iPhotonIDTextureWidth + l)+2] = 0xFF;
				pBuffer[4*(k*m_iPhotonIDTextureWidth + l)+3] = 0xFF;
			}
		}

		//printf("found %d\n", iFoundPhotons);
		pBuffer = m_pPhotonIDBuffer + 4 * (m_iPhotonIDTextureWidth * PHOTONID_SQUARE_SIZE * iY + PHOTONID_SQUARE_SIZE * iX);
		fTexCoordX += 1.0f / iSquaresPerDimension;
		if(fTexCoordX >= 1)
		{
			fTexCoordY += 1.0f / iSquaresPerDimension;
			fTexCoordX = 0;
		}

		iX++;
		if(iX == m_iPhotonIDTextureWidth/PHOTONID_SQUARE_SIZE)
		{
			iY ++;
			iX = 0;
		}
	}
	END_TIMING
}

void PhotonMap::printIDTexture()
{
	int i, j;
	for(i=0; i<m_iPhotonIDTextureHeight; i++)
	{
		for(j=0; j<m_iPhotonIDTextureWidth; j++)
		{
			unsigned int value = *((unsigned int*)&m_pPhotonIDBuffer[4*(i*m_iPhotonIDTextureWidth+j)]);
			unsigned char a = (value & 0xFF000000) >> 24;
			unsigned char b = (value & 0x00FF0000) >> 16;
			unsigned char c = (value & 0x0000FF00) >> 8;
			unsigned char d = (value & 0x000000FF);
			printf("0x%8x | ", value);
		}
		printf("\n");
	}
	printf("-------------------\n");
}

void PhotonMap::printPowTexture()
{
	int i, j;
	for(i=0; i<m_iTextureWidth; i++)
	{
		for(j=0; j<m_iTextureHeight; j++)
		{
			unsigned int value = *((unsigned int*)&m_pPhotonPowerBuffer[4*(i*m_iPhotonIDTextureWidth+j)]);
			unsigned char a = (value & 0xFF000000) >> 24;
			unsigned char b = (value & 0x00FF0000) >> 16;
			unsigned char c = (value & 0x0000FF00) >> 8;
			unsigned char d = (value & 0x000000FF);
			printf("0x%8x | ", value);
		}
		printf("\n");
	}
	printf("-------------------\n");
}

void PhotonMap::printDirTexture()
{
	int i, j;
	for(i=0; i<m_iTextureWidth; i++)
	{
		for(j=0; j<m_iTextureHeight; j++)
		{
			unsigned int value = *((unsigned int*)&m_pPhotonDirBuffer[4*(i*m_iPhotonIDTextureWidth+j)]);
			unsigned char a = (value & 0xFF000000) >> 24;
			unsigned char b = (value & 0x00FF0000) >> 16;
			unsigned char c = (value & 0x0000FF00) >> 8;
			unsigned char d = (value & 0x000000FF);
			printf("0x%8x | ", value);
		}
		printf("\n");
	}
	printf("-------------------\n");
}