#ifndef _PHOTON_H_
#define _PHOTON_H_

#include "geometry.h"

#define MAX_LIGHTS_IN_PHOTON_MAP 10

typedef enum
{
	POINT_LIGHT,
	CONE_LIGHT,
	AREA_LIGHT
}LightType;

class Photon
{
public:
	Photon() {};

	void trace(Mesh *pMesh);

	float m_fRed, m_fGreen, m_fBlue;
	float m_fX, m_fY, m_fZ;
	float m_fDirX, m_fDirY, m_fDirZ;
//	float m_fTheta, m_fPhi;
	float m_fOriginX, m_fOriginY, m_fOriginZ;
};

class Light
{
public:
	Light() {};
	Light(float fRed, float fGreen, float fBlue, 
		float fX, float fY, float fZ,
		LightType eType);
	void emitPhotons(Photon *pPhotons);
	void setNumberPhotons(int numberPhotons){ m_iNumberPhotons = numberPhotons; }

private:
	void emitPhotonsDiffuseLight(Photon *pPhotons);
	void emitPhotonsDirectionalLight(Photon *pPhotons, float fDirX, float fDirY, float fDirZ, float fWide);

public:
	float m_fRed, m_fGreen, m_fBlue;
	LightType m_eType;
	int m_iNumberPhotons;
	float m_fX, m_fY, m_fZ;
};

class PhotonMap
{
public:
	PhotonMap() {};
	~PhotonMap();
	PhotonMap(int numberLights, Light **lights, int numberPhotons);

	void trace(Mesh *pMesh, unsigned int iNPasses);
	void generatePhotons();
	void buildTextures();
	void buildPhotonIdTexture(Mesh *pMesh);
	void addMesh(Mesh *pMesh);

	void setNumberPhotons(int iNumberPhotons);

	char *getPowerTexture() { return m_pPhotonPowerBuffer; }
	char *getDirTexture() { return m_pPhotonDirBuffer; }
	char *getIDTexture() { return m_pPhotonIDBuffer; }
	int getTextureWidth() { return m_iTextureWidth; }
	int getTextureHeight() { return m_iTextureHeight; }
	int getPhotonIDTextureWidth() { return m_iPhotonIDTextureWidth; }
	int getPhotonIDTextureHeight() { return m_iPhotonIDTextureHeight; }
	float *getTexCoord() { return m_pfTexCoordBuffer; }
	int getNumberPhotons() { return m_iNumberPhotons; }
	Photon *getPhoton(int i) const { return &m_pPhotons[i]; }

public:
	//debug functions
	void printIDTexture();
	void printPowTexture();
	void printDirTexture();

private:

	Photon *m_pPhotons;
	Light *m_pLights[MAX_LIGHTS_IN_PHOTON_MAP];
	int m_iNumberLights;
	int m_iNumberPhotons;
	int m_iPhotonsPerLight;
	int m_iTextureWidth;
	int m_iTextureHeight;
	int m_iPhotonIDTextureWidth;
	int m_iPhotonIDTextureHeight;

	char *m_pPhotonIDBuffer;
	char *m_pPhotonDirBuffer;
	char *m_pPhotonPowerBuffer;

	float *m_pfTexCoordBuffer;
};

#endif