#ifndef _VECTOR_H_
#define _VECTOR_H_

#include <math.h>

struct Vector3 {
	float x, y, z;       

	Vector3(void) {}

	Vector3(float a, float b, float c)
	{
		x=a;            
		y=b;
		z=c;            
	}

	float length()
	{
		return sqrt(x*x + y*y + z*z);
	}

	float dist(Vector3 v1)
	{
		float dist = sqrt((x - v1.x) * (x - v1.x) + (y - v1.y) * (y - v1.y) + (z - v1.z) * (z - v1.z));
		return dist;
	}

	void normalize()
	{
		float fL = length();
		x = x / fL;
		y = y / fL;
		z = z / fL;
	}

	friend Vector3 operator-(Vector3 &v1, Vector3 &v2);
	friend Vector3 operator+(Vector3 &v1, Vector3 &v2);
	friend Vector3 operator/(const Vector3 &v, float a);
	friend Vector3 operator*(const Vector3 &v, float a);
	friend Vector3 operator*(float a, const Vector3 &v);

	Vector3& operator-=(Vector3 &v2);
	Vector3& operator/=(float a);
	Vector3& operator*=(float a);

	Vector3 operator-();
}; 

inline Vector3 unitVector(Vector3 a)
{
	float length = a.length();
	return a/length;
}

inline Vector3 cross(Vector3 v1, Vector3 v2)
{
	Vector3 temp;

	temp.x = v1.y*v2.z - v1.z*v2.y;
	temp.y = v1.z*v2.x - v1.x*v2.z;
	temp.z = v1.x*v2.y - v1.y*v2.x;

	return temp;
}

inline Vector3 operator/(const Vector3 &v, float a)
{
	return Vector3(v.x/a, v.y/a, v.z/a);
}

inline Vector3 operator-(Vector3 &v1, Vector3 &v2)
{
	Vector3 v3;
	v3.x = v1.x - v2.x;
	v3.y = v1.y - v2.y;
	v3.z = v1.z - v2.z;
	return v3;
}

inline Vector3 operator+(Vector3 &v1, Vector3 &v2)
{
	Vector3 v3;
	v3.x = v1.x + v2.x;
	v3.y = v1.y + v2.y;
	v3.z = v1.z + v2.z;
	return v3;
}

inline Vector3 operator*(const Vector3 &v, float a)
{
	return Vector3(v.x*a, v.y*a, v.z*a);
}

inline Vector3 operator*(float a, const Vector3 &v)
{
	return Vector3(v.x*a, v.y*a, v.z*a);
}

inline Vector3& Vector3::operator/=(float a)
{
	*this = *this / a;
	return *this;
}

inline Vector3& Vector3::operator-=(Vector3 &v2)
{
	*this = *this - v2;
	return *this;
}

inline Vector3& Vector3::operator*=(float a)
{
	*this = *this * a;
	return *this;
}
inline Vector3 Vector3::operator-()
{
	return Vector3(-x, -y, -z);
}

#endif